"""Django Approval Workflow Package.

A powerful, flexible Django package for implementing dynamic multi-step
approval workflows in your Django applications.
"""

__version__ = "0.8.5"
__author__ = "Mohamed Salah"
__email__ = "info@codxi.com"

default_app_config = "approval_workflow.apps.ApprovalWorkflowConfig"
