# Generated by Django 5.2.4 on 2025-10-04 05:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("approval_workflow", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="approvalinstance",
            name="approval_type",
            field=models.CharField(
                choices=[
                    ("approve", "Approve"),
                    ("submit", "Submit with Form"),
                    ("check_in_verify", "Check-in/Verify"),
                    ("move", "Move"),
                ],
                default="approve",
                help_text="Type of approval action (approve, submit, check-in/verify, move)",
                max_length=20,
            ),
        ),
    ]
